/*******Stata code combining raw survey tabulations 1905-2015 and tax data corrections*****/

#delimit;
clear;
set more off;

cd "D:\Users\t.piketty\Dropbox\WIDRussia\NPZ2017DistributionSeries\";





/****Step 1: importing corrected Pareto coefficients****/ 
import excel NPZ2017AppendixB.xlsx, sheet("TB0") cellrange(A8:I119) firstrow clear;
keep year y yf ynf b;
rename year year2;
rename b bt;
save Gpinter\IncomeParameters.dta, replace;


/*****Step 2: applying Pareto coefficients corrections for all years 1980-2015**/
/*****applying corrections for years 1980-2015*****/
foreach year in 1980 1985 1988 1989
{;
clear;
import excel Gpinter\Input\RawSurveyTabulationsUSSR.xlsx, sheet("`year'") firstrow;
keep year country component average p thr b;
append using Gpinter\IncomeParameters.dta;
keep if year2==. | year2==`year';
egen btop=max(bt);
replace b=btop if thr==2100 & `year'==1980;
drop if thr>2100 & `year'==1980;
replace b=btop if thr==2400 & `year'==1985;
drop if thr>2400 & `year'==1985;
replace b=btop if thr==3000 & (`year'==1988 | `year'==1989);
keep year country component average p thr b;
replace component="yf" if _n==1;
replace country="Russia" if _n==1;
export excel using Gpinter\Input\FinalParetoTabulations.xlsx, sheet("`year'yf") sheetreplace firstrow(variables);
clear;
};
foreach year in 1990 1991 1992 1993
{;
clear;
import excel Gpinter\Input\RawSurveyTabulationsHBS.xlsx, sheet("`year'RU") firstrow;
keep year country component average p thr;
append using Gpinter\IncomeParameters.dta;
keep if year2==. | year2==`year';
egen btop=max(bt);
gen b=btop if thr==600 & (`year'==1990 | `year'==1991);
drop if (thr==200 | thr>600) & (`year'==1990 | `year'==1991);
replace thr=0.9*thr if p<0.5 & (`year'==1990 | `year'==1991);
replace b=btop if thr==7500 & `year'==1992;
drop if (thr==500 | thr==1000 |  thr>7500) & `year'==1992;
replace b=btop if thr==100000 & `year'==1993;
drop if (thr==10000 | thr==15000 | thr>100000) & `year'==1993;
replace thr=12*thr;
keep year country component average p thr b;
replace component="yf" if _n==1;
replace country="Russia" if _n==1;
export excel using Gpinter\Input\FinalParetoTabulations.xlsx, sheet("`year'yf") sheetreplace firstrow(variables);
clear;
};
forvalues year=1994/2015
{;
clear;
import excel Gpinter\Output\RawSurveySeriesRLMS.xlsx, sheet("survey income, Russia, `year'") firstrow;
append using Gpinter\IncomeParameters.dta;
keep if year2==. | year2==`year';
egen btop=max(bt);
replace p=round(100000*p);
drop if p>90000;
replace bracketavg=btop*thr if p==90000;
gsort -p;
replace topavg=bracketavg if _n==1;
replace topavg=((100000-p[_n-1])*topavg[_n-1]+(p[_n-1]-p)*bracketavg)/(100000-p) if _n>1;
gsort p;
replace average=topavg if _n==1;
replace b=topavg/thr;
replace p=p/100000;
rename bracketavg bracketavgno;
keep year country component average p thr topavg bracketavgno b;
drop if (p>0 & p<0.1) | (p>0.1 & p<0.2);
replace component="yf" if _n==1;
replace country="Russia" if _n==1;
export excel using Gpinter\Input\FinalParetoTabulations.xlsx, sheet("`year'yf") sheetreplace firstrow(variables);
clear;
};



/******Step 3: keeping benchmark tax corrections for years 2008-2015****/
/*****Applying Pareto coefficient corrections for 1980-2008***/
/*****And integrating 1905-1976 benchmark estimates***/
foreach year in 1905 1928 1934 1956 1959 1961 1964 1966 1968 1972 1976
{;
clear;
import excel Gpinter\Input\RawTabulationsEmplSurvey.xlsx, sheet("`year'") firstrow;
replace component="yf" if _n==1;
replace country="Russia" if _n==1;
export excel using Gpinter\Input\FinalTabulations.xlsx, sheet("`year'yf") sheetreplace firstrow(variables);
clear;
};
foreach year in 1980 1985 1988 1989
{;
clear;
import excel Gpinter\Input\RawSurveyTabulationsUSSR.xlsx, sheet("`year'") firstrow;
keep year country component average p thr b;
append using Gpinter\IncomeParameters.dta;
keep if year2==. | year2==`year';
egen btop=max(bt);
replace b=btop if thr==2100 & `year'==1980;
drop if thr>2100 & `year'==1980;
replace b=btop if thr==2400 & `year'==1985;
drop if thr>2400 & `year'==1985;
replace b=btop if thr==3000 & (`year'==1988 | `year'==1989);
keep year country component average p thr b;
replace component="yf" if _n==1;
replace country="Russia" if _n==1;
export excel using Gpinter\Input\FinalTabulations.xlsx, sheet("`year'yf") sheetreplace firstrow(variables);
clear;
};
foreach year in 1990 1991 1992 1993
{;
clear;
import excel Gpinter\Input\RawSurveyTabulationsHBS.xlsx, sheet("`year'RU") firstrow;
keep year country component average p thr;
append using Gpinter\IncomeParameters.dta;
keep if year2==. | year2==`year';
egen btop=max(bt);
gen b=btop if thr==600 & (`year'==1990 | `year'==1991);
drop if (thr==200 | thr>600) & (`year'==1990 | `year'==1991);
replace thr=0.9*thr if p<0.5 & (`year'==1990 | `year'==1991);
replace b=btop if thr==7500 & `year'==1992;
drop if (thr==500 | thr==1000 |  thr>7500) & `year'==1992;
replace b=btop if thr==100000 & `year'==1993;
drop if (thr==10000 | thr==15000 | thr>100000) & `year'==1993;
replace thr=12*thr;
keep year country component average p thr b;
replace component="yf" if _n==1;
replace country="Russia" if _n==1;
export excel using Gpinter\Input\FinalTabulations.xlsx, sheet("`year'yf") sheetreplace firstrow(variables);
clear;
};


forvalues year=1994/2007
{;
clear;
import excel Gpinter\Output\RawSurveySeriesRLMS.xlsx, sheet("survey income, Russia, `year'") firstrow;
append using Gpinter\IncomeParameters.dta;
keep if year2==. | year2==`year';
egen btop=max(bt);
replace p=round(100000*p);
drop if p>90000;
replace bracketavg=btop*thr if p==90000;
gsort -p;
replace topavg=bracketavg if _n==1;
replace topavg=((100000-p[_n-1])*topavg[_n-1]+(p[_n-1]-p)*bracketavg)/(100000-p) if _n>1;
gsort p;
replace average=topavg if _n==1;
replace b=topavg/thr;
replace p=p/100000;
rename bracketavg bracketavgno;
keep year country component average p thr topavg bracketavgno b;
drop if (p>0 & p<0.1) | (p>0.1 & p<0.2);
replace component="yf" if _n==1;
replace country="Russia" if _n==1;
export excel using Gpinter\Input\FinalTabulations.xlsx, sheet("`year'yf") sheetreplace firstrow(variables);
clear;
};
forvalues year=2008/2015
{;
clear;
import excel Gpinter\Input\CorrectedSurveyTabulationsRLMSTax.xlsx, sheet("`year'v21") firstrow;
replace component="yf" if _n==1;
export excel using Gpinter\Input\FinalTabulations.xlsx, sheet("`year'yf") sheetreplace firstrow(variables);
clear;
};









