/*******Stata code combining RLMS survey tabulations 1994-2015 and corrections from income tax tabulations*****/

#delimit;
clear;
set more off;

cd "D:\Dropbox\WIDRussia\NPZ2017DistributionSeries\";


/****Step 1: apply gpinter to Gpinter/Input/RawSurveyTabulationsRLMS.xlsx and save output in Gpinter/Output/RawSurveySeriesRLMS.xlsx***/


/****Step 2: importing raw survey output and applying corrections****/ 
import excel NPZ2017AppendixB.xlsx, sheet("TB13") cellrange(A4:AA10) firstrow;
save Gpinter/IncomeTaxCorrectionsFactors.dta, replace;
clear;

/*****applying corrections for variants 1/1-21 (Pareto variants)*****/
forvalues year=1994/2015
{;
forvalues variant=15/35 
{;
import excel Gpinter\Output\RawSurveySeriesRLMS.xlsx, sheet("survey income, Russia, `year'") firstrow;
gen bvariant=`variant'/10;
replace p=round(100000*p);
drop if p>90000;
replace bracketavg=bvariant*thr if p==90000;
gsort -p;
replace topavg=bracketavg if _n==1;
replace topavg=((100000-p[_n-1])*topavg[_n-1]+(p[_n-1]-p)*bracketavg)/(100000-p) if _n>1;
gsort p;
replace average=topavg if _n==1;
replace b=topavg/thr;
replace p=p/100000;
replace component="v1`variant'" if _n==1;
rename bracketavg bracketavgno;
keep year country component average p thr topavg bracketavgno b bvariant;
drop if (p>0 & p<0.1) | (p>0.1 & p<0.2);
export excel using Gpinter\Input\CorrectedSurveyTabulationsRLMSPareto.xlsx, sheet("`year'v1`variant'") sheetreplace firstrow(variables);
clear;
};
};


forvalues year=2008/2015
{;
/*****applying corrections for variants 2/1-5 (tax-data declaration-rates variants*****/
forvalues variant=1/5 
{;
import excel Gpinter\Output\RawSurveySeriesRLMS.xlsx, sheet("survey income, Russia, `year'") firstrow;
append using Gpinter/IncomeTaxCorrectionsFactors.dta;
replace n`year'=n`year'/fdec`variant';
gen p1p=1-(n`year'[128]+n`year'[129]+n`year'[130]+n`year'[131]+n`year'[132])/n`year'[133];
gen p2p=1-(n`year'[129]+n`year'[130]+n`year'[131]+n`year'[132])/n`year'[133];
gen p3p=1-(n`year'[130]+n`year'[131]+n`year'[132])/n`year'[133];
replace p1p=round(100000*p1p);
replace p2p=round(100000*p2p);
replace p3p=round(100000*p3p);
replace p=round(100000*p);
gen p1pp=p if p1p>p & p1p<=p[_n+1];
egen p1=max(p1pp);
gen p2pp=p if p2p>p & p2p<=p[_n+1];
egen p2=max(p2pp);
gen p3pp=p if p3p>p & p3p<=p[_n+1];
egen p3=max(p3pp);
gen p0=int(p1/10000)*10000;
gen ratio1p=(10000000*max(0.05,1-rded`year'[128]*rded0[128]))/thr if p==p1;
egen ratio1=max(ratio1p);
replace ratio1=max(1,ratio1);
gen ratio2p=(100000000*max(0.05,1-rded`year'[129]*rded0[129]))/thr if p==p2;
egen ratio2=max(ratio2p);
replace ratio2=max(1,ratio2);
gen ratio3p=(500000000*max(0.05,1-rded`year'[130]*rded0[130]))/thr if p==p3;
egen ratio3=max(ratio3p);
replace ratio3=max(1,ratio3);
gen n0p=_n if p==p0;
egen n0=max(n0p);
gen n1p=_n if p==p1;
egen n1=max(n1p);
gen n2p=_n if p==p2;
egen n2=max(n2p);
gen n3p=_n if p==p3;
egen n3=max(n3p);
gen ratio=1 if _n==1;
replace ratio=1 if p<=p0;
replace ratio=ratio[_n-1]+(ratio1-1)/(n1-n0) if p>p0 & p<=p1;
replace ratio=ratio[_n-1]+(ratio2-ratio1)/(n2-n1) if p>p1 & p<=p2;
replace ratio=ratio[_n-1]+(ratio3-ratio2)/(n3-n2) if p>p2 & p<=p3;
replace ratio=ratio[_n-1] if p>p3;
drop if _n>127;
replace thr=thr*ratio;
replace bracketavg=bracketavg*(ratio+ratio[_n+1])/2 if _n<127;
replace bracketavg=bracketavg*ratio if _n==127;
gsort -p;
replace topavg=bracketavg if _n==1;
replace topavg=((100000-p[_n-1])*topavg[_n-1]+(p[_n-1]-p)*bracketavg)/(100000-p) if _n>1;
gsort p;
replace average=topavg if _n==1;
replace b=topavg/thr;
replace p=p/100000;
rename bracketavg bracketavgno;
replace component="v2`variant'" if _n==1;
keep year country component average p thr topavg bracketavgno b p1 p2 p3 ratio1 ratio2 ratio3 ratio;
drop if (p>0 & p<0.1) | (p>0.1 & p<0.2);
export excel using Gpinter\Input\CorrectedSurveyTabulationsRLMSTax.xlsx, sheet("`year'v2`variant'") sheetreplace firstrow(variables);
clear;
};

/*****applying corrections for variants 3/1-4 (tax-data deduction-rates variants*****/
forvalues variant=1/4 
{;
import excel Gpinter\Output\RawSurveySeriesRLMS.xlsx, sheet("survey income, Russia, `year'") firstrow;
append using Gpinter/IncomeTaxCorrectionsFactors.dta;
replace n`year'=n`year'/fdec1;
gen p1p=1-(n`year'[128]+n`year'[129]+n`year'[130]+n`year'[131]+n`year'[132])/n`year'[133];
gen p2p=1-(n`year'[129]+n`year'[130]+n`year'[131]+n`year'[132])/n`year'[133];
gen p3p=1-(n`year'[130]+n`year'[131]+n`year'[132])/n`year'[133];
replace p1p=round(100000*p1p);
replace p2p=round(100000*p2p);
replace p3p=round(100000*p3p);
replace p=round(100000*p);
gen p1pp=p if p1p>p & p1p<=p[_n+1];
egen p1=max(p1pp);
gen p2pp=p if p2p>p & p2p<=p[_n+1];
egen p2=max(p2pp);
gen p3pp=p if p3p>p & p3p<=p[_n+1];
egen p3=max(p3pp);
gen p0=int(p1/10000)*10000;
gen ratio1p=(10000000*max(0.05,1-rded`year'[128]*rded`variant'[128]))/thr if p==p1;
egen ratio1=max(ratio1p);
replace ratio1=max(1,ratio1);
gen ratio2p=(100000000*max(0.05,1-rded`year'[129]*rded`variant'[129]))/thr if p==p2;
egen ratio2=max(ratio2p);
replace ratio2=max(1,ratio2);
gen ratio3p=(500000000*max(0.05,1-rded`year'[130]*rded`variant'[130]))/thr if p==p3;
egen ratio3=max(ratio3p);
replace ratio3=max(1,ratio3);
gen n0p=_n if p==p0;
egen n0=max(n0p);
gen n1p=_n if p==p1;
egen n1=max(n1p);
gen n2p=_n if p==p2;
egen n2=max(n2p);
gen n3p=_n if p==p3;
egen n3=max(n3p);
gen ratio=1 if _n==1;
replace ratio=1 if p<=p0;
replace ratio=ratio[_n-1]+(ratio1-1)/(n1-n0) if p>p0 & p<=p1;
replace ratio=ratio[_n-1]+(ratio2-ratio1)/(n2-n1) if p>p1 & p<=p2;
replace ratio=ratio[_n-1]+(ratio3-ratio2)/(n3-n2) if p>p2 & p<=p3;
replace ratio=ratio[_n-1] if p>p3;
drop if _n>127;
replace thr=thr*ratio;
replace bracketavg=bracketavg*(ratio+ratio[_n+1])/2 if _n<127;
replace bracketavg=bracketavg*ratio if _n==127;
gsort -p;
replace topavg=bracketavg if _n==1;
replace topavg=((100000-p[_n-1])*topavg[_n-1]+(p[_n-1]-p)*bracketavg)/(100000-p) if _n>1;
gsort p;
replace average=topavg if _n==1;
replace b=topavg/thr;
replace p=p/100000;
rename bracketavg bracketavgno;
replace component="v3`variant'" if _n==1;
keep year country component average p thr topavg bracketavg b p1 p2 p3 ratio1 ratio2 ratio3 ratio;
drop if (p>0 & p<0.1) | (p>0.1 & p<0.2);
export excel using Gpinter\Input\CorrectedSurveyTabulationsRLMSTax.xlsx, sheet("`year'v3`variant'") sheetreplace firstrow(variables);
clear;
};
};


/****Step 3: apply gpinter to Gpinter/Input/CorrectedSurveyTabulationsRLMSPareto.xlsx and CorrectedSurveyTabulationsRLMSTax.xlsx and save output in Gpinter/Output/CorrectedSurveySeriesRLMSPareto.xlsx and CorrectedSurveySeriesRLMSTax.xlsx***/

